<?php
/**
 * Mateo Theme functions and definitions
 *
 * @link https://developer.wordpress.org/themes/basics/theme-functions/
 * @package mateo
 */

// phpcs:ignore
require_once __DIR__ . '/vendor/autoload.php';

define( 'MATEO_THEME_ABSPATH', __DIR__ );
define( 'MATEO_VERSION', '1.0.3' );
define( 'MATEO_MAX_CONTENT_WIDTH', 940 );
define( 'MATEO_CSS_STRIP_EOL', true );
define( 'MATEO_STYLE_ID', 'mateo-style' );


if ( ! isset( $content_width ) ) {
	// phpcs:ignore --WPThemeReview.CoreFunctionality.PrefixAllGlobals.NonPrefixedVariableFound
	$content_width = MATEO_MAX_CONTENT_WIDTH;
}

if ( ! function_exists( 'mateo_setup' ) ) :
	/**
	 * Sets up theme defaults and registers support for various WordPress features.
	 *
	 * Note that this function is hooked into the after_setup_theme hook, which
	 * runs before the init hook. The init hook is too late for some features, such
	 * as indicating support for post thumbnails.
	 */
	function mateo_setup() {
		/*
		 * Make theme available for translation.
		 * Translations can be filed in the /languages/ directory.
		 * If you're building a theme based on mateo, use a find and replace
		 * to change 'mateo' to the name of your theme in all the template files.
		 */
		load_theme_textdomain( 'mateo', get_template_directory() . '/languages' );

		// Add default posts and comments RSS feed links to head.
		add_theme_support( 'automatic-feed-links' );

		/*
		 * Let WordPress manage the document title.
		 * By adding theme support, we declare that this theme does not use a
		 * hard-coded <title> tag in the document head, and expect WordPress to
		 * provide it for us.
		 */
		add_theme_support( 'title-tag' );

		/*
		 * Enable support for Post Thumbnails on posts and pages.
		 *
		 * @link https://developer.wordpress.org/themes/functionality/featured-images-post-thumbnails/
		 */
		add_theme_support( 'post-thumbnails' );
		set_post_thumbnail_size( 825, 300 );

		// This theme uses wp_nav_menu() in one location.
		register_nav_menus(
			array(
				/* translators: primary menu */
				'menu-1' => esc_html__( 'Primary', 'mateo' ),
			)
		);

		/*
		 * Switch default core markup for search form, comment form, and comments
		 * to output valid HTML5.
		 */
		add_theme_support(
			'html5',
			array(
				'search-form',
				'comment-form',
				'comment-list',
				'gallery',
				'caption',
			)
		);

		// Add theme support for selective refresh for widgets.
		add_theme_support( 'customize-selective-refresh-widgets' );

		/**
		 * Add support for core custom logo.
		 *
		 * @link https://codex.wordpress.org/Theme_Logo
		 */
		add_theme_support(
			'custom-logo',
			array(
				'height'      => 64,
				'width'       => 64,
				'flex-width'  => true,
				'flex-height' => true,
			)
		);

		// Add support for Block Styles.
		add_theme_support( 'wp-block-styles' );
	}
endif;
add_action( 'after_setup_theme', 'mateo_setup' );

/**
 * Register widget area.
 *
 * @link https://developer.wordpress.org/themes/functionality/sidebars/#registering-a-sidebar
 */
function mateo_widgets_init() {
	register_sidebar(
		array(
			'name'          => esc_html__( 'Sidebar', 'mateo' ),
			'id'            => 'sidebar-1',
			'description'   => esc_html__( 'Add widgets here.', 'mateo' ),
			'before_widget' => '<section id="%1$s" class="widget %2$s">',
			'after_widget'  => '</section>',
			'before_title'  => '<h3 class="widget-title">',
			'after_title'   => '</h3>',
		)
	);

	register_sidebar(
		array(
			/* translators: Footer widgets area */
			'name'          => esc_html__( 'Footer', 'mateo' ),
			'id'            => 'footer-1',
			'description'   => esc_html__( 'Add widgets here.', 'mateo' ),
			'before_widget' => '<section id="%1$s" class="widget %2$s">',
			'after_widget'  => '</section>',
			'before_title'  => '<h3 class="widget-title">',
			'after_title'   => '</h3>',
		)
	);

	register_sidebar(
		array(
			/* translators: Centered footer widgets */
			'name'          => esc_html__( 'Footer centered', 'mateo' ),
			'id'            => 'footer-2',
			'description'   => esc_html__( 'Add widgets here.', 'mateo' ),
			'before_widget' => '<section id="%1$s" class="widget %2$s">',
			'after_widget'  => '</section>',
			'before_title'  => '<h3 class="widget-title">',
			'after_title'   => '</h3>',
		)
	);
}

add_action( 'widgets_init', 'mateo_widgets_init' );

/**
 * Add some required settings before other theme scripts are loaded.
 *
 * @return string
 */
function mateo_js_config() {
	$text = array(
		'show'       => esc_attr__( 'Show menu', 'mateo' ),
		'hide'       => esc_attr__( 'Hide menu', 'mateo' ),
		'showParent' => esc_attr__( 'Show parent menu', 'mateo' ),
		'previous'   => esc_attr__( 'Previous', 'mateo' ),
		'next'       => esc_attr__( 'Next', 'mateo' ),
	);

	return 'window.Mateo = {}; Mateo.labels = ' . wp_json_encode( $text ) . ';';
}
/**
 * Enqueue scripts and styles.
 */
function mateo_scripts() {
	$assets_root = get_stylesheet_directory_uri() . '/assets';
	// CSS.
	wp_enqueue_style( 'mateo-fonts', 'https://fonts.googleapis.com/css?family=Roboto:300,400,500,700|Material+Icons', array() ); // phpcs:ignore
	wp_enqueue_style( MATEO_STYLE_ID, get_stylesheet_uri(), array(), MATEO_VERSION );
	wp_style_add_data( MATEO_STYLE_ID, 'rtl', 'replace' );
	// JS.
	wp_enqueue_script( 'jquery-core', false, array(), MATEO_VERSION, true );
	wp_add_inline_script( 'jquery-core', mateo_js_config() );
	wp_enqueue_script( 'mateo-custom', $assets_root . '/js/theme.min.js', array( 'jquery-core' ), MATEO_VERSION, true );

	if ( is_singular() && comments_open() && get_option( 'thread_comments' ) ) {
		wp_enqueue_script( 'comment-reply' );
	}

}

add_action( 'wp_enqueue_scripts', 'mateo_scripts' );

/**
 * Custom template tags for this theme.
 */
require __DIR__ . '/inc/template-tags.php';

/**
 * Functions which enhance the theme by hooking into WordPress.
 */
require __DIR__ . '/inc/template-functions.php';

/**
 * Customizer additions.
 */
require __DIR__ . '/inc/customizer.php';

/**
 * Load Jetpack compatibility file.
 */
if ( defined( 'JETPACK__VERSION' ) ) {
	require __DIR__ . '/inc/jetpack.php';
}

/**
 * Load WooCommerce compatibility file.
 */
if ( class_exists( 'WooCommerce' ) ) {
	require __DIR__ . '/inc/woocommerce.php';
}
