<?php
/**
 * Theme Customizer functions.
 *
 * @package mateo
 */

use Mateo\ThemeColor;

if ( ! function_exists( 'mateo_custom_css' ) ) {
	/**
	 * Generate custom CSS.
	 *
	 * @param string $id If present the function will return css rules wrapped into <style> tag, if not only css rules will be returned.
	 *
	 * @return string
	 */
	function mateo_custom_css( $id = '' ) {
		$theme_color = new ThemeColor( mateo_get_color_scheme() );
		$theme_color->init_user_settings();
		// are we in preview mode ?
		if ( is_customize_preview() ) {
			$theme_color->init_change_set();
		}

		return $id ? $theme_color->get_style_tag( $id ) : $theme_color->get_css( MATEO_CSS_STRIP_EOL );
	}
}

if ( ! function_exists( 'mateo_display_blog_name' ) ) {
	/**
	 * Displays the blog name.
	 */
	function mateo_display_blog_name() {
		bloginfo( 'name' );
	}
}

if ( ! function_exists( 'mateo_display_blog_description' ) ) {
	/**
	 * Displays the blog description.
	 */
	function mateo_display_blog_description() {
		bloginfo( 'description' );
	}
}

/**
 * Add postMessage support for site title and description for the Theme Customizer.
 *
 * @param WP_Customize_Manager $wp_customize Theme Customizer object.
 */
function mateo_customize_register( WP_Customize_Manager $wp_customize ) {

	$theme_color = new ThemeColor( mateo_get_color_scheme() );

	// Configure WP default settings.
	$wp_customize->get_setting( 'blogname' )->transport        = 'postMessage';
	$wp_customize->get_setting( 'blogdescription' )->transport = 'postMessage';

	// Configure selective refresh for defined settings.
	$wp_customize->selective_refresh->add_partial(
		'blogname',
		array(
			'selector'        => '.site-title a',
			'render_callback' => 'mateo_display_blog_name',
		)
	);
	$wp_customize->selective_refresh->add_partial(
		'blogdescription',
		array(
			'selector'        => '.site-description',
			'render_callback' => 'mateo_display_blog_description',
		)
	);
	// Add color scheme setting and control.
	$wp_customize->add_setting(
		'color_scheme',
		array(
			'default'           => 'default',
			'sanitize_callback' => 'mateo_sanitize_color_scheme',
			'transport'         => 'postMessage',
		)
	);

	$wp_customize->add_control(
		'color_scheme',
		array(
			'label'    => _x( 'Base Color Scheme', 'Color palette', 'mateo' ),
			'section'  => 'colors',
			'type'     => 'select',
			'choices'  => mateo_get_color_scheme_choices(),
			'priority' => 1,
		)
	);
	// Primary color.
	$wp_customize->add_setting(
		'primary',
		array(
			'default'           => $theme_color->get_color( 'primary' ),
			'sanitize_callback' => 'sanitize_hex_color',
			'transport'         => 'postMessage',
		)
	);
	$wp_customize->add_control(
		new WP_Customize_Color_Control(
			$wp_customize,
			'primary',
			array(
				'label'   => _x( 'Primary color', 'Color palette', 'mateo' ),
				'section' => 'colors',
			)
		)
	);

	// Light version of primary color.
	$wp_customize->add_setting(
		'primaryLight',
		array(
			'default'           => $theme_color->get_color( 'primaryLight' ),
			'sanitize_callback' => 'sanitize_hex_color',
			'transport'         => 'postMessage',
		)
	);
	$wp_customize->add_control(
		new WP_Customize_Color_Control(
			$wp_customize,
			'primaryLight',
			array(
				'label'   => _x( 'Primary light color', 'Color palette', 'mateo' ),
				'section' => 'colors',
			)
		)
	);

	// Dark version of primary color.
	$wp_customize->add_setting(
		'primaryDark',
		array(
			'default'           => $theme_color->get_color( 'primaryDark' ),
			'sanitize_callback' => 'sanitize_hex_color',
			'transport'         => 'postMessage',
		)
	);
	$wp_customize->add_control(
		new WP_Customize_Color_Control(
			$wp_customize,
			'primaryDark',
			array(
				'label'   => _x( 'Primary dark color', 'Color palette', 'mateo' ),
				'section' => 'colors',
			)
		)
	);
	// WP core feature required by review team.
	// Turn it into a shortcut for the text on primary settings.
	$wp_customize->get_setting( 'header_textcolor' )->transport   = 'postMessage';
	$wp_customize->get_setting( 'header_textcolor' )->default     = $theme_color->get_color( 'textOnPrimary' );
	$wp_customize->get_control( 'header_textcolor' )->description = _x( 'Shortcut for the "Text on primary" option', 'Color palette', 'mateo' );
	// Text color on primary.
	$wp_customize->add_setting(
		'textOnPrimary',
		array(
			'default'           => $theme_color->get_color( 'textOnPrimary' ),
			'sanitize_callback' => 'sanitize_hex_color',
			'transport'         => 'postMessage',
		)
	);
	$wp_customize->add_control(
		new WP_Customize_Color_Control(
			$wp_customize,
			'textOnPrimary',
			array(
				'label'   => _x( 'Text on primary', 'Color palette', 'mateo' ),
				'section' => 'colors',
			)
		)
	);
	// Text color on primary.
	$wp_customize->add_setting(
		'link',
		array(
			'default'           => $theme_color->get_color( 'link' ),
			'sanitize_callback' => 'sanitize_hex_color',
			'transport'         => 'postMessage',
		)
	);
	$wp_customize->add_control(
		new WP_Customize_Color_Control(
			$wp_customize,
			'link',
			array(
				'label'   => _x( 'Link color', 'Color palette', 'mateo' ),
				'section' => 'colors',
			)
		)
	);

	// Content background.
	$wp_customize->add_setting(
		'contentBgr',
		array(
			'default'           => $theme_color->get_color( 'contentBgr' ),
			'sanitize_callback' => 'sanitize_hex_color',
			'transport'         => 'postMessage',
		)
	);
	$wp_customize->add_control(
		new WP_Customize_Color_Control(
			$wp_customize,
			'contentBgr',
			array(
				'label'   => _x( 'Text background', 'Color palette', 'mateo' ),
				'section' => 'colors',
			)
		)
	);

	// Page background color.
	$wp_customize->add_setting(
		'pageBgr',
		array(
			'default'           => $theme_color->get_color( 'pageBgr' ),
			'sanitize_callback' => 'sanitize_hex_color',
			'transport'         => 'postMessage',
		)
	);
	$wp_customize->add_control(
		new WP_Customize_Color_Control(
			$wp_customize,
			'pageBgr',
			array(
				'label'   => _x( 'Page background', 'Color palette', 'mateo' ),
				'section' => 'colors',
			)
		)
	);

	// Add main text color.
	$wp_customize->add_setting(
		'textBody',
		array(
			'default'           => $theme_color->get_color( 'textBody' ),
			'sanitize_callback' => 'sanitize_hex_color',
			'transport'         => 'postMessage',
		)
	);
	$wp_customize->add_control(
		new WP_Customize_Color_Control(
			$wp_customize,
			'textBody',
			array(
				'label'   => _x( 'Text color', 'Color palette', 'mateo' ),
				'section' => 'colors',
			)
		)
	);

	// Add main text color.
	$wp_customize->add_setting(
		'title',
		array(
			'default'           => $theme_color->get_color( 'title' ),
			'sanitize_callback' => 'sanitize_hex_color',
			'transport'         => 'postMessage',
		)
	);
	$wp_customize->add_control(
		new WP_Customize_Color_Control(
			$wp_customize,
			'title',
			array(
				'label'       => _x( 'Title color', 'Color palette', 'mateo' ),
				'section'     => 'colors',
				'description' => 'The color is applied only if site or post featured image is not displayed.',
			)
		)
	);
}

add_action( 'customize_register', 'mateo_customize_register', 11 );

/**
 * Set up the WordPress core custom header feature.
 */
function mateo_custom_header_setup() {
	add_theme_support(
		'custom-header',
		apply_filters(
			'mateo_custom_header_args',
			array(
				'default-image'      => get_template_directory_uri() . '/assets/img/header.jpg',
				'header_text'        => false,
				'default-text-color' => 'fff',
				'width'              => 1920,
				'height'             => 1080,
				'flex-height'        => true,
				'flex-width'         => true,
			)
		)
	);
}

add_action( 'after_setup_theme', 'mateo_custom_header_setup' );


/**
 * Outputs an Underscore template for generating CSS for the color scheme.
 *
 * The template generates the css dynamically for instant display in the
 * Customizer preview.
 */
function mateo_color_scheme_css_template() {
	$theme_color = new ThemeColor( mateo_get_color_scheme() );
	?>
	<script type="text/html" id="tmpl-mateo-color-scheme">
		<?php
			// phpcs:ignore
			echo $theme_color->get_css_template( MATEO_CSS_STRIP_EOL );
		?>
	</script>
	<?php
}

add_action( 'customize_controls_print_footer_scripts', 'mateo_color_scheme_css_template' );


/**
 * Binds the JS listener to make Customizer color_scheme control.
 *
 * Passes color scheme data as mateoColorScheme global.
 */
function mateo_customize_control_js() {
	wp_enqueue_script(
		'mateo-color-scheme-control',
		get_template_directory_uri() . '/assets/js/color-scheme-control.min.js',
		array(
			'customize-controls',
			'iris',
			'underscore',
			'wp-util',
		),
		MATEO_VERSION,
		true
	);
	wp_localize_script( 'mateo-color-scheme-control', 'mateoColorScheme', mateo_get_color_schemes() );
}

add_action( 'customize_controls_enqueue_scripts', 'mateo_customize_control_js' );

/**
 * Binds JS handlers to make the Customizer preview reload changes asynchronously.
 */
function mateo_customize_preview_js() {
	wp_enqueue_script(
		'mateo-customize-preview',
		get_template_directory_uri() . '/assets/js/customize-preview.min.js',
		array( 'customize-preview' ),
		MATEO_VERSION,
		true
	);
}

add_action( 'customize_preview_init', 'mateo_customize_preview_js' );


if ( ! function_exists( 'mateo_sanitize_color_scheme' ) ) :
	/**
	 * Handles sanitization for the theme color schemes.
	 *
	 * Create your own mateo_sanitize_color_scheme() function to override
	 * in a child theme.
	 *
	 * @param string $value Color scheme name value.
	 *
	 * @return string Color scheme name.
	 */
	function mateo_sanitize_color_scheme( $value ) {
		$color_schemes = mateo_get_color_scheme_choices();

		if ( ! array_key_exists( $value, $color_schemes ) ) {
			return 'default';
		}

		return $value;
	}
endif;

/**
 * Enqueues front-end CSS for color scheme.
 *
 * @see wp_add_inline_style()
 */
function mateo_color_scheme_css() {
	wp_add_inline_style( MATEO_STYLE_ID, mateo_custom_css() );
}

add_action( 'wp_enqueue_scripts', 'mateo_color_scheme_css' );


/**
 *
 * Can be filtered with {@see 'mateo_color_schemes'}.
 *
 * @return array An associative array of color scheme options.
 */
function mateo_get_color_schemes() {
	$schemes = array(
		'default'   => array(
			/* translators: default color scheme */
			'label'  => _x( 'Default', 'Color palette', 'mateo' ), // Pink.
			'colors' => array(
				'primary'       => '#e91e63',
				'primaryLight'  => '#ff6090',
				'primaryDark'   => '#b0003a',
				'textOnPrimary' => '#ffffff',
				'link'          => '#e91e63',
				'textBody'      => '#212121',
				'title'         => '#212121',
				'contentBgr'    => '#ffffff',
				'pageBgr'       => '#ededed',
			),
		),
		'red'       => array(
			'label'  => _x( 'Red', 'Color palette', 'mateo' ),
			'colors' => array(
				'primary'       => '#d32f2f',
				'primaryLight'  => '#ff6659',
				'primaryDark'   => '#9a0007',
				'textOnPrimary' => '#ffffff',
				'link'          => '#d32f2f',
				'textBody'      => '#333333',
				'title'         => '#333333',
				'contentBgr'    => '#ffffff',
				'pageBgr'       => '#ededed',
			),
		),
		'purple'    => array(
			'label'  => _x( 'Purple', 'Color palette', 'mateo' ),
			'colors' => array(
				'primary'       => '#8e24aa',
				'primaryLight'  => '#c158dc',
				'primaryDark'   => '#5c007a',
				'textOnPrimary' => '#ffffff',
				'link'          => '#8e24aa',
				'textBody'      => '#333333',
				'title'         => '#333333',
				'contentBgr'    => '#ffffff',
				'pageBgr'       => '#ededed',
			),
		),
		'blue'      => array(
			'label'  => _x( 'Blue', 'Color palette', 'mateo' ),
			'colors' => array(
				'primary'       => '#2196f3',
				'primaryLight'  => '#6ec6ff',
				'primaryDark'   => '#0069c0',
				'textOnPrimary' => '#ffffff',
				'link'          => '#2196f3',
				'textBody'      => '#333333',
				'title'         => '#333333',
				'contentBgr'    => '#ffffff',
				'pageBgr'       => '#ededed',
			),
		),
		'orange'    => array(
			'label'  => _x( 'Orange', 'Color palette', 'mateo' ),
			'colors' => array(
				'primary'       => '#ff5722',
				'primaryLight'  => '#ff8a50',
				'primaryDark'   => '#c41c00',
				'textOnPrimary' => '#ffffff',
				'link'          => '#ff5722',
				'textBody'      => '#333333',
				'title'         => '#333333',
				'contentBgr'    => '#ffffff',
				'pageBgr'       => '#ededed',
			),
		),
		'green'     => array(
			'label'  => _x( 'Green', 'Color palette', 'mateo' ),
			'colors' => array(
				'primary'       => '#43a047',
				'primaryLight'  => '#76d275',
				'primaryDark'   => '#00701a',
				'textOnPrimary' => '#ffffff',
				'link'          => '#43a047',
				'textBody'      => '#333333',
				'title'         => '#333333',
				'contentBgr'    => '#ffffff',
				'pageBgr'       => '#ededed',
			),
		),
		'blue_grey' => array(
			'label'  => _x( 'Blue Grey', 'Color palette', 'mateo' ),
			'colors' => array(
				'primary'       => '#546e7a',
				'primaryLight'  => '#819ca9',
				'primaryDark'   => '#29434e',
				'textOnPrimary' => '#ffffff',
				'link'          => '#546e7a',
				'textBody'      => '#333333',
				'title'         => '#333333',
				'contentBgr'    => '#ffffff',
				'pageBgr'       => '#ededed',
			),
		),
		'dark'      => array(
			'label'  => _x( 'Dark', 'Color palette', 'mateo' ),
			'colors' => array(
				'primary'       => '#333333',
				'primaryLight'  => '#5c5c5c',
				'primaryDark'   => '#0c0c0c',
				'textOnPrimary' => '#ffffff',
				'link'          => '#333333',
				'textBody'      => '#333333',
				'title'         => '#333333',
				'contentBgr'    => '#ffffff',
				'pageBgr'       => '#ededed',
			),
		),
	);

	return apply_filters( 'mateo_color_schemes', $schemes );
}

if ( ! function_exists( 'mateo_get_color_scheme' ) ) {
	/**
	 * Retrieves the current color scheme.
	 *
	 * Create your own mateo_get_color_scheme() function to override in a child theme.
	 *
	 * @return array An associative array of either the current or default color scheme HEX values.
	 */
	function mateo_get_color_scheme() {
		$color_scheme_option = get_theme_mod( 'color_scheme', 'default' );
		$color_schemes       = mateo_get_color_schemes();

		if ( array_key_exists( $color_scheme_option, $color_schemes ) ) {
			return $color_schemes[ $color_scheme_option ]['colors'];
		}

		return $color_schemes['default']['colors'];
	}
}

if ( ! function_exists( 'mateo_get_color_scheme_choices' ) ) {
	/**
	 * Retrieves an array of color scheme choices registered for Twenty Sixteen.
	 *
	 * Create your own mateo_get_color_scheme_choices() function to override
	 * in a child theme.
	 *
	 * @return array Array of color schemes.
	 */
	function mateo_get_color_scheme_choices() {
		$color_schemes                = mateo_get_color_schemes();
		$color_scheme_control_options = array();

		foreach ( $color_schemes as $color_scheme => $value ) {
			$color_scheme_control_options[ $color_scheme ] = $value['label'];
		}

		return $color_scheme_control_options;
	}
}


